package speiger.src.coreengine.rendering.shader;

import org.lwjgl.opengl.GL20;

import speiger.src.collections.objects.lists.ObjectArrayList;
import speiger.src.collections.objects.lists.ObjectList;
import speiger.src.coreengine.assets.base.IAssetProvider;
import speiger.src.coreengine.assets.base.IManagedAsset;
import speiger.src.coreengine.assets.base.IReloadableAsset.ISimpleRealodableAsset;

public class ShaderTracker implements ISimpleRealodableAsset, IManagedAsset {
	ObjectList<Shader<?>> shaders = new ObjectArrayList<>();
	int shader;
	IAssetProvider provider;
		
	public void registerShader(Shader<?> shader) {
		shaders.add(shader);
		shader.load(provider);
	}
	
	public void bind(ShaderProgram program) {
		bind(shaderId(program));
	}
	
	public void bind(int shaderId) {
		if(shader == shaderId) return;
		shader = shaderId;
		GL20.glUseProgram(shaderId);
	}
	
	public void unbind() {
		if(shader == 0) return;
		shader = 0;
		GL20.glUseProgram(0);
	}
	
	public boolean isActive() { return shader != 0; }
	public int getActiveShader() { return shader; }
	
	private int shaderId(ShaderProgram program) {
		return program == null ? 0 : program.id();
	}
		
	@Override
	public void destroy() {
		shaders.forEach(Shader::remove);
		shaders.clear();
		shader = 0;
	}

	@Override
	public String getName() { return "shader_tracker"; }
	@Override
	public void setProvider(IAssetProvider provider) { this.provider = provider; }
	@Override
	public void onAssetsReloaded(IAssetProvider provider) {
		ShaderProgram.IMPORTS.clear();
		shaders.forEach(provider, this::load);
		shader = 0;
	}
	
	private void load(IAssetProvider provider, Shader<?> shader) {
		shader.load(provider);
	}
}
